#!/usr/bin/env node
"use strict";

var _mri = _interopRequireDefault(require("mri"));

var _execa = _interopRequireDefault(require("execa"));

var fs = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var _chalk = _interopRequireDefault(require("chalk"));

var _resolve = _interopRequireDefault(require("resolve"));

var _changeCase = require("change-case");

var _prompts = _interopRequireDefault(require("prompts"));

var _resolveAsBin = _interopRequireDefault(require("resolve-as-bin"));

var _getModularRoot = _interopRequireDefault(require("./getModularRoot"));

var _preflightCheck = _interopRequireDefault(require("./preflightCheck"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// Makes the script crash on unhandled rejections instead of silently
// ignoring them. In the future, promise rejections that are not handled will
// terminate the Node.js process with a non-zero exit code.
// See https://github.com/facebook/create-react-app/blob/f36d61a/packages/react-scripts/bin/react-scripts.js#L11-L16
process.on('unhandledRejection', err => {
  throw err;
});
const cracoBin = (0, _resolveAsBin.default)('craco');
const jestBin = (0, _resolveAsBin.default)('jest');
const cracoConfig = path.join(__dirname, '..', 'DO_NOT_IMPORT_THIS_OR_YOU_WILL_BE_FIRED_craco.config.js');

function execSync(file, args, options = {
  log: true
}) {
  const {
    log,
    ...opts
  } = options;

  if (log) {
    console.log(_chalk.default.grey(`$ ${file} ${args.join(' ')}`));
  }

  return _execa.default.sync(file, args, {
    stdin: process.stdin,
    stderr: process.stderr,
    stdout: process.stdout,
    cleanup: true,
    ...opts
  });
}

function isModularType(dir, type) {
  const packageJsonPath = path.join(dir, 'package.json');

  if (fs.existsSync(packageJsonPath)) {
    var _packageJson$modular;

    const packageJson = fs.readJsonSync(packageJsonPath);
    return ((_packageJson$modular = packageJson.modular) === null || _packageJson$modular === void 0 ? void 0 : _packageJson$modular.type) === type;
  }

  return false;
} // recursively get all files in a folder


function getAllFiles(dirPath, arrayOfFiles = []) {
  const files = fs.readdirSync(dirPath);
  files.forEach(function (file) {
    const pathToCheck = path.join(dirPath, file);

    if (fs.statSync(pathToCheck).isDirectory()) {
      arrayOfFiles = getAllFiles(pathToCheck, arrayOfFiles);
    } else {
      arrayOfFiles.push(pathToCheck);
    }
  });
  return arrayOfFiles;
}

async function run() {
  const help = `
  Usage:

    $ modular add <package-name>
    $ modular start
    $ modular build
    $ modular test
`;
  await (0, _preflightCheck.default)();
  const argv = (0, _mri.default)(process.argv.slice(2));
  const command = argv._[0];

  try {
    switch (command) {
      case 'add':
        return addPackage(argv._[1], argv['unstable-type']);

      case 'test':
        return test(process.argv.slice(3));

      case 'start':
        return start(argv._[1]);

      case 'build':
        return buildSequential(argv._[1].split(','), argv['preserve-modules']);

      default:
        console.log(help);
        process.exit(1);
    }
  } catch (err) {
    /* eslint-disable @typescript-eslint/no-unsafe-member-access */
    // When there is an `execa` Error with an `.exitCode` the
    // error message will already have been printed out by the
    // process so we only need to bail with the exit code.
    if (err.exitCode !== undefined) {
      process.exit(err.exitCode);
    }
    /* eslint-enable @typescript-eslint/no-unsafe-member-access */


    throw err;
  }
}

async function addPackage(name, typeArg) {
  const type = typeArg || (await (0, _prompts.default)([{
    name: 'type',
    type: 'select',
    message: `What kind of package is ${name}?`,
    choices: [{
      title: 'A plain package',
      value: 'package'
    }, {
      title: 'A view within an application',
      value: 'view'
    }, {
      title: 'A standalone application',
      value: 'app'
    }],
    initial: 0
  }])).type;
  const modularRoot = (0, _getModularRoot.default)();
  const newPackageName = (0, _changeCase.paramCase)(name);
  const newComponentName = (0, _changeCase.pascalCase)(name);
  const newPackagePath = path.join(modularRoot, 'packages', newPackageName);
  const packageTypePath = path.join(__dirname, '../types', type); // create a new package source folder

  if (fs.existsSync(newPackagePath)) {
    console.error(`The package named ${name} already exists!`);
    process.exit(1);
  }

  fs.mkdirpSync(newPackagePath);
  fs.copySync(packageTypePath, newPackagePath);
  const packageFilePaths = getAllFiles(newPackagePath);

  for (const packageFilePath of packageFilePaths) {
    fs.writeFileSync(packageFilePath, fs.readFileSync(packageFilePath, 'utf8').replace(/PackageName__/g, newPackageName).replace(/ComponentName__/g, newComponentName));
  }

  execSync('yarnpkg', [], {
    cwd: newPackagePath
  });
}

function test(args) {
  if (process.env.SKIP_PREFLIGHT_CHECK !== 'true') {
    const verifyPackageTree = require('react-scripts/scripts/utils/verifyPackageTree'); // eslint-disable-line @typescript-eslint/no-var-requires


    verifyPackageTree();
  }

  const modularRoot = (0, _getModularRoot.default)();
  let argv = process.argv.slice(3).concat(['--config', path.join(__dirname, '..', 'jest-config.js')]); // Watch unless on CI or explicitly running all tests

  if (!process.env.CI && args.indexOf('--watchAll=false') === -1) {
    // https://github.com/facebook/create-react-app/issues/5210
    argv.push('--watchAll');
  } // via https://github.com/facebook/create-react-app/blob/master/packages/react-scripts/scripts/test.js
  // This is a very dirty workaround for https://github.com/facebook/jest/issues/5913.
  // We're trying to resolve the environment ourselves because Jest does it incorrectly.
  // TODO: remove this as soon as it's fixed in Jest.


  function resolveJestDefaultEnvironment(name) {
    const jestDir = path.dirname(_resolve.default.sync('jest', {
      basedir: __dirname
    }));
    const jestCLIDir = path.dirname(_resolve.default.sync('jest-cli', {
      basedir: jestDir
    }));
    const jestConfigDir = path.dirname(_resolve.default.sync('jest-config', {
      basedir: jestCLIDir
    }));
    return _resolve.default.sync(name, {
      basedir: jestConfigDir
    });
  }

  const cleanArgv = [];
  let env = 'jsdom';
  let next;

  do {
    var _next;

    next = argv.shift();

    if (next === '--env') {
      env = argv.shift();
    } else if (((_next = next) === null || _next === void 0 ? void 0 : _next.indexOf('--env=')) === 0) {
      env = next.substring('--env='.length);
    } else {
      cleanArgv.push(next);
    }
  } while (argv.length > 0); // @ts-ignore


  argv = cleanArgv;
  let resolvedEnv;

  try {
    resolvedEnv = resolveJestDefaultEnvironment(`jest-environment-${env}`);
  } catch (e) {// ignore
  }

  if (!resolvedEnv) {
    try {
      resolvedEnv = resolveJestDefaultEnvironment(env);
    } catch (e) {// ignore
    }
  }

  const testEnvironment = resolvedEnv || env;
  argv.push('--env', testEnvironment || ''); // ends the section copied from CRA

  return execSync(jestBin, argv, {
    cwd: modularRoot,
    log: false,
    // @ts-ignore
    env: {
      BABEL_ENV: 'test',
      NODE_ENV: 'test',
      PUBLIC_URL: '',
      MODULAR_ROOT: modularRoot
    }
  });
}

function start(appPath) {
  const modularRoot = (0, _getModularRoot.default)();

  if (!isModularType(path.join(modularRoot, 'packages', appPath), 'app')) {
    throw new Error(`The package at ${appPath} is not a valid modular app.`);
  }

  execSync(cracoBin, ['start', '--config', cracoConfig], {
    cwd: path.join(modularRoot, 'packages', appPath),
    log: false,
    // @ts-ignore
    env: {
      MODULAR_ROOT: modularRoot
    }
  });
} // run builds sequentially


async function buildSequential(directoryNames, preserveModules) {
  console.log('building packages:', directoryNames.join(', '));

  for (let i = 0; i < directoryNames.length; i++) {
    try {
      await build(directoryNames[i], preserveModules);
    } catch (err) {
      console.error(`building ${directoryNames[i]} failed`);
      throw err;
    }
  }
}

async function build(directoryName, preserveModules) {
  const modularRoot = (0, _getModularRoot.default)();

  if (isModularType(path.join(modularRoot, 'packages', directoryName), 'app')) {
    // create-react-app doesn't support plain module outputs yet,
    // so --preserve-modules has no effect here
    await fs.remove(`dist/${directoryName}`); // TODO: this shouldn't be sync

    execSync(cracoBin, ['build', '--config', cracoConfig], {
      cwd: path.join(modularRoot, 'packages', directoryName),
      log: false,
      // @ts-ignore
      env: {
        MODULAR_ROOT: modularRoot
      }
    });
    await fs.move(`packages/${directoryName}/build`, `dist/${directoryName}`);
  } else {
    // it's a view/package, run a library build
    const {
      build
    } = await Promise.resolve().then(() => _interopRequireWildcard(require('./build'))); // ^ we do a dynamic import here to defer the module's initial side effects
    // till when it's actually needed (i.e. now)

    await build(directoryName, preserveModules);
  }
}

void run().catch(err => {
  // todo - cleanup on errors
  console.error(err);
  process.exit(1);
});